#include "editor.h"

//{{{
typedef struct menu_entry
{
	const char *str;
	const char *hotkey;
	void (*func)(void);
} menu_entry;
//}}}
//{{{
typedef struct menu_info
{
	const char *str;
	int xpos, highlight_width, width;
	int num_entries;
	menu_entry *entries;
} menu_info;
//}}}

#define ICON_CTRL  "\x7F"
#define ICON_SHIFT "\x1B"

//{{{
menu_entry file_menu[] = {
	{ "New",          ICON_SHIFT "N", file_new },
	{ "Open",         ICON_SHIFT "O", file_open },
	{ "Save",         ICON_SHIFT "S", file_save },
	{ "Save as",      "", file_save_as },
	{ "Exit",         "ESC", file_exit}
	};
//}}}
//{{{
menu_entry edit_menu[] = {
	{ "Cut",          ICON_SHIFT "X", edit_cut },
	{ "Copy",         ICON_SHIFT "Y", edit_copy },
	{ "Paste",        "=", edit_paste },
	{ "Clear",        "",  edit_clear },
	{ "Select all",   ICON_SHIFT "A", edit_select_all }
	};
//}}}
//{{{
menu_entry transform_menu[] = {
	{ "Flip horizontal", ICON_SHIFT "H", transform_mirror_horizontal },
	{ "Flip vertical",   ICON_SHIFT "V", transform_mirror_vertical },
	{ "Rotate 180deg",   ICON_SHIFT "R", transform_rotate_180 },
	{ "Cut corners",     ICON_SHIFT "C", transform_smooth }
	};
//}}}
//{{{
menu_entry map_menu[] = {
	{ "Properties",   "MODE",           map_properties   },
	{ "Resize",       "",               map_resize       },
	{ "Next map",     "+",              map_next         },
	{ "Previous map", "-",              map_previous     },
	{ "Insert map",   "",               map_insert       },
	{ "Delete",       "",               map_delete       },
	{ "Move up",      ICON_CTRL "\x17", map_move_up      },
	{ "Move down",    ICON_CTRL "\x18", map_move_down    },
	{ "Import",       "",               map_import_level }
	};
//}}}

#define NUM_MENUS 4
//{{{
menu_info menus[NUM_MENUS] = {
	{ "File",      11, 17, 45, 5, file_menu },
	{ "Edit",      28, 17, 55, 5, edit_menu },
	{ "Transform", 45, 39, 70, 4, transform_menu },
	{ "Map",       84, 17, 65, 9, map_menu }
	};
//}}}

int open_menu = 0;
int menu_pos = 0;

//{{{
void enter_menu_state(int menu)
{
	input_state = menu_state;
	open_menu = menu;
	menu_pos = 0;
	draw_menu_labels();
	menu_toggle_highlight(menu);
	draw_open_menu();
}
//}}}
//{{{
void draw_menu_labels(void)
{
	int ii;
	for(ii=0; ii<NUM_MENUS; ii++)
		draw_string(menus[ii].str, menus[ii].xpos+1, 2);
}
//}}}
//{{{
void undraw_open_menus(void)
{
	draw_map_field();
}
//}}}
//{{{
void draw_open_menu(void)
{
	int ii;
	int xpos = menus[open_menu].xpos;
	const char *str;
	menu_info *menu = &menus[open_menu];
	draw_rect(xpos, 9, menu->width+2,
	          menu->num_entries*7+2);
	for(ii=0; ii<menu->num_entries; ii++)
	{
		// Option
		draw_string(menu->entries[ii].str, xpos+2, 11+7*ii);
		// Hotkey (right-aligned)
		str = menu->entries[ii].hotkey;
		draw_string(str, xpos+menu->width-1-DrawStrWidth(str, F_4x6), 11+7*ii);
	}
	xor_rect(xpos+1, 10+menu_pos*7, menus[open_menu].width, 7);
}
//}}}
//{{{
void menu_toggle_highlight(int menu)
{
	xor_rect(menus[menu].xpos, 0, menus[menu].highlight_width, 9);
}
//}}}

//{{{
void switch_menu(int menu)
{
	menu_toggle_highlight(open_menu);
	undraw_open_menus();
	open_menu = menu;
	menu_toggle_highlight(open_menu);
	menu_pos = 0;
	draw_open_menu();
}
//}}}
//{{{
void menu_state(int key)
{
	if(key == KEY_RIGHT) {
		switch_menu((open_menu+1)%NUM_MENUS);
	} else if(key == KEY_LEFT) {
		if(open_menu==0) switch_menu(NUM_MENUS-1);
		else             switch_menu(open_menu-1);
	} else if(key == KEY_DOWN) {
		menu_pos = (menu_pos+1) % menus[open_menu].num_entries;
		draw_open_menu();
	} else if(key == KEY_UP) {
		if(menu_pos==0) menu_pos = menus[open_menu].num_entries - 1;
		else menu_pos--;
		draw_open_menu();
	} else if(key == KEY_ENTER) {
		undraw_open_menus();
		menus[open_menu].entries[menu_pos].func();
		if(input_state == menu_state) // If the menu didn't change state
			enter_edit_state();
	}
	else if(key == KEY_ESC) enter_edit_state();
	else if(key == KEY_F1) enter_palette_state();
	else if(key == KEY_F2) switch_menu(0);
	else if(key == KEY_F3) switch_menu(1);
	else if(key == KEY_F4) switch_menu(2);
	else if(key == KEY_F5) switch_menu(3);
}
//}}}


