#include "engine.h"

char active_set[32];

//{{{
int load_kyelib(int levelnum)
{
	FILE *fin = NULL;
	unsigned short offset;
	file_header header;
	
	fin = fopen(active_set, "rb");
	if(!fin) goto error_return;
	
	if( fread_fast(&header, sizeof(file_header), 1, fin) < 1 )
		goto error_return;
	
	if( memcmp(header.magic_bytes, "SKye", 4) ) {
		// Nope, not going to support DC-Kye levels, since it's so easy to
		// convert them.
		goto error_return;
	}
	
	if(header.revision != 2) {
		// Only the most recent version of the SKye level spec is supported
		// by the game engine
		goto error_return;
	}
	
	if(levelnum > header.num_levels)
		goto error_return;
	
	if( fseek_fast(fin, sizeof(unsigned short)*levelnum+sizeof(file_header),
	    SEEK_SET) != 0 )
		goto error_return;
	if( fread_fast(&offset, sizeof(unsigned short), 1, fin) < 1 )
		goto error_return;
	if( fseek_fast(fin, offset, SEEK_SET) != 0 )
		goto error_return;
	
	if(!load_skye_level(fin))
		goto error_return;
	
	fclose(fin);
	return 1;
	
error_return:
	if(fin) fclose(fin);
	return 0;
}
//}}}

//{{{
int load_skye_level(FILE *fin)
{
	int ii, jj;
	map_r2 loaded_level;
	if( fread_fast(&loaded_level, sizeof(loaded_level), 1, fin) < 1 ) return 0;
	set_level_size(loaded_level.width, loaded_level.height);
//	current_map.gamerules = loaded_level.gamerules;
	current_map.gamerules = RULESET_KYE92;
	freadstr(current_map.name,    32, fin);
	freadstr(current_map.author,  32, fin);
	freadstr(current_map.hint,    64, fin);
	freadstr(current_map.congrat, 64, fin);
	for(ii=0; ii<current_map.height; ii++)
	{
		memset(current_map.t[ii], 0,
		       sizeof(engine_map_square)*current_map.width);
		for(jj=0; jj<current_map.width; jj++)
			current_map.t[ii][jj].v = fgetc(fin);
	}
	return 1;
}
//}}}
//{{{
void set_level_size(int width, int height)
{
	engine_map_square **pos, *alloc_pos;
	size_t size = sizeof(engine_map_square*)*height +
	              sizeof(engine_map_square)*width*height;
	int ii;
	
	if(current_map.width == width && current_map.height == height)
		return;
	
	if(current_map.t) {
		free(current_map.t);
		current_map.t = NULL;
	}
	
	// Pool allocation of all the rows together
	current_map.t = pos = (engine_map_square**)malloc_throw(size);
	memset(pos, 0, size);
	alloc_pos = (engine_map_square*)(pos + height);
	
	for(ii=0; ii<height; ii++)
	{
		*pos = alloc_pos;
		pos++;
		alloc_pos += width;
	}
	
	current_map.width = width;
	current_map.height = height;
}
//}}}
